/**************************************************************************\
*                                                                          *
*                       Panel para dibujar la marcha exacta                *
*                           --------------------                           *
*                                                                          *
*       Se deforma el dibujo para que se vean mejor las lentes:            *
*       Se dibujan las dimensiones horizontales a escala 1/2.              *
*       Las verticales a escala 4/1.                                       *
*       El radio de curvatura a escala 1/2.                                *
*                                                                          *
*                                                                          *
\**************************************************************************/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;


//=============================================================================


public class DibujoExactoFC extends Canvas
{
    int     eje = 105;
    int     numColores;
    int     numSup;
    int     nir;



    //--------------------  Funcin para dibujar    ---------------------------

    public void paint (Graphics graphics)
    {
        nir = ExactoFC.nir; 
        numColores = ExactoFC.numColores;
        numSup = FuncionesExactasFC.numSup;


                        //  Utiliza las funciones de Graphics2D
     
        Graphics2D gr = (Graphics2D) graphics;

                        //  Dibuja las lentes y diafragmas

        dibujaElementos (gr);


                        //  Si no hay objeto definido no dibuja nada ms

        if (SistemaFC.existeObjeto[0] == 0) 
            return;

        if (ExactoFC.dibuja == 0)
            return;                   

                        //  Dibuja los rayos meridianos

        dibujaHazRayos (gr);


                        //  Dibuja el plaano Imagen

        dibujaPlanoImagen (gr);

    }



    //===============     Dibujo de lentes y diafragmas  ======================


    void dibujaElementos (Graphics gr)
    {
        int     i, n, y, z1, z2;
        double  d, z, ze1, ze2, zb1, zb2;
        double  radioDib1, radioDib2, altura;


        gr.setColor (Color.yellow);

        for (i = 1; i < 7; i ++)
        {
            if (SistemaFC.tipoE[i] == 1)             //  Es una lente
            {
                if (SistemaFC.valPE[i] != 0.)        //  Existe
                {
                    radioDib1 = ExactoFC.radioDib1[i]; 
                    radioDib2 = ExactoFC.radioDib2[i]; 
                    altura = ExactoFC.diamMax / 2.;

                    z = SistemaFC.valZE[i];
                    d = ExactoFC.espesor[i];
                    ze1 = z - d / 2.;
                    ze2 = z + d / 2.;

                    zb1 = ze1 + ExactoFC.valDib1PE[i];
                    zb2 = ze2 + ExactoFC.valDib2PE[i];

                                    //  Dibuja la superficie anterior

                    dibujaSuperficie (gr, ze1, radioDib1, altura);
    
                                    //  Dibuja la superficie posterior

                    dibujaSuperficie (gr, ze2, radioDib2, altura);

                                    //  Dibuja los bordes
                    
                    z1 = (int) (zb1 / 2.);
                    z2 = (int) (zb2 / 2.);

                    y = eje - (int) altura * 2 - 4;
                    gr.drawLine (z1, y, z2, y);

                    y = eje + (int) altura * 2 + 4;
                    gr.drawLine (z1, y, z2, y);
                    
                    ExactoFC.dzb1[i] = zb1 - z;
                    ExactoFC.dzb2[i] = zb2 - z;
                }
            }
            else                                        //  Es diafragma
            {
                if (SistemaFC.valDE[i] != ExactoFC.diamMax)  //  Existe
                {
                    z = SistemaFC.valZE[i];
                    altura = SistemaFC.valDE[i] / 2.;
                    dibujaDiafragma (gr, z, altura);
                }
            }
        }
    }


    //===============     Dibujo de una superficie       ======================

    void dibujaSuperficie (Graphics gr, double xv, double radio, double altura)
    {

        int  x, y, ancho, h;
        double  xvDib, alturaDib, radioDib;
        double  zeta, alfa1, alfa2, xc;

        //  Para mejor visualizacin del dibujo
        //  se reducen las dimensiones horizontales a la mitad.
        //  Las verticales se multiplican por 2.
        //  El radio se multiplica por 12,

        xvDib = xv / 2.;
        alturaDib = altura * 2. + 4.;
        radioDib = radio * 12.;
        xc = xvDib + radioDib;

        x = (int) xvDib;

                                    //  Es superficie plana
        if (radio >= 1.e10)
        {
            h = (int) (alturaDib);
            gr.drawLine (x, eje - h, x, eje + h);
            return;
        }

                                    //  Es superficie curva
        zeta = Math.asin (alturaDib / radioDib);

        if (radioDib > 0)
        {
            alfa1 = 3.141592 - zeta;
            alfa2 = alfa1 + 2. * zeta;
        }

        else
        {
            alfa1 = zeta;
            alfa2 = alfa1 - 2. * zeta;
        }
 

        dibujaArco (gr, xc, radioDib, alfa1, alfa2);

        return;
    }


    //===============     Dibujo de un arco              ======================

    void dibujaArco (Graphics gr, double xc, double radio,
                      double alfa1, double alfa2)

                //  Los ngulos estn en radianes
    {
        int     i, n;
        int     x[] = new int[1000];
        int     y[] = new int[1000];
        double  alfa, delta;

        delta = 2. / Math.abs (radio);

        for (i = 0, n = 0, alfa = alfa1; alfa <= alfa2 && i < 1000;
                          i ++, alfa += delta)
        {
            if (radio > 0)
                x[i] = (int) (xc + radio * Math.cos (alfa));
            else
                x[i] = (int) (xc - radio * Math.cos (alfa));
            y[i] = eje - (int) (radio * Math.sin (alfa));
        }

        n = i;

        for (i = 1; i < n; i ++)
        {
            gr.drawLine (x[i-1], y[i-1], x[i], y[i]);
        }
    }



    //===============     Dibujo de un diafragma         ======================

    void dibujaDiafragma (Graphics gr, double xv, double altura)
    {
        int  x, r;

        x = (int) (xv / 2.);
        r = (int) (altura * 2.);
        gr.drawLine (x, eje - 73, x, eje - r);
        gr.drawLine (x, eje + 73, x, eje + r);
    }


    //===============     Dibujo de los rayos meridianos  =====================


    void dibujaHazRayos (Graphics gr)
    {
        int     i, j, k, l, n, ia, ip, i1, i2, z, y, w, z1, z2, h, y1, y2, obj;
        double  radioDib, zv, altura;
        Color naranja = new Color (255, 128, 0);
        Color   colorRayos[] = { Color.green, Color.darkGray, Color.red };
        

        gr.setColor (Color.white);


        for (obj = 0; obj < 2; obj ++)
        {
            for (k = numColores - 1; k >= 0; k --)
            {
                for (j = 0; j <= nir * 2; j ++)
                {
                    for (i = 0; i <= numSup; i ++)
                    {

                           //  Tramo de la superficie i a la i+1

                                //  Extremo izquierdo del tramo

                        if (i == 0)
                        {
                                    //  Tramo del objeto a la 1 superficie
                 
                            if (numColores == 1)
                                gr.setColor (Color.green);
                            else
                                gr.setColor (Color.white);

                                    //  Objeto lejano o virtual
                 
                            if ((SistemaFC.zObjeto[0] < 0) ||
                                (SistemaFC.real[0] == 0) ||
                                (SistemaFC.zObjeto[0] >= 1.e10))
                     
                            {
                                z1 = 0;
                                y1 = eje - 
                                   (int) (FuncionesExactasFC.dmRayo[obj][k][i+1][j] * 2.);
                            }
                            else
                                    //  Objeto real prximo
                            {
                                z1 = (int) (SistemaFC.zObjeto[0] / 2.);
                                if (obj == 0)
                                    y1 = eje;
                                else
                                    y1 = eje -
                                       (int) (SistemaFC.valYObj[0] * 2);
                            }
                 
                        }
                 
                                    //  Restantes tramos
                     
                        else
                        {
                            if (numColores == 1)
                                gr.setColor (Color.green);
                            else
                                gr.setColor (colorRayos[k]);

                            if (FuncionesExactasFC.pasam[obj][k][i][j] == 0)
                                break;
                 
                            z1 = (int) (FuncionesExactasFC.zmRayo[obj][k][i][j] / 2.);
                            y1 = eje - 
                               (int) (FuncionesExactasFC.ymRayo[obj][k][i][j] * 2.);

                                    //  Para mezclar los colores en el ojo

                            if (y1 >= eje)
                            {
                                if (k == 1)     y1 ++;
                                if (k == 2)     y1 --;
                            }
                            else
                            {
                                if (k == 1)     y1 --;
                                if (k == 2)     y1 ++;
                            }
                        }


                                //  Extremo derecho del tramo

                        z2 = (int) (FuncionesExactasFC.zmRayo[obj][k][i+1][j] / 2.);
                        y2 = eje - (int) (FuncionesExactasFC.ymRayo[obj][k][i+1][j] * 2.);

                        gr.drawLine (z1, y1, z2, y2);
                    }
                }
            }
        }
    }


    //===============     Dibujo el plano imagen        =====================


    void dibujaPlanoImagen (Graphics gr)
    {
        int     i, j, k, obj, h, zd, x, y;
        double  zi;


        Color naranja = new Color (255, 128, 0);


                        //  Dibuja el plano de imagen

        h = 70;
        zi = FuncionesExactasFC.imagParax + ExactoFC.valZImag;
        zd = (int) (zi / 2.);

        gr.setColor (Color.magenta);

        gr.drawLine (zd, eje - h, zd, eje + h);
    }


    //=========================================================================
                                                          
    public void redraw ()
    {
        repaint ();
    }
}

