/**************************************************************************\
*                                                                          *
*          Clase que contiene las funciones de clculo paraxial            *
*                                                                          *
\**************************************************************************/


import java.awt.*;

public class FuncionesParaxialesFC
{


    /**********************************************************************\
    *                                                                      *
    *                   Funciones del trazado de rayos                     *    
    *                                                                      *
    \**********************************************************************/

    //=====================     tablaElementos  ============================

    //  Crea la tabla de elementos que existen 

    static int     tablaElementos ()
    {
        int     i = 0, n;

        AnteproyectoFC.elemento[0] = 0;
        AnteproyectoFC.numUltimaLente = 0;
        AnteproyectoFC.numUltimoElemento = 0;

        for (n = 1; n < 7; n ++)
        {
            if (SistemaFC.existe[n] == 1) 
            {
                AnteproyectoFC.numUltimoElemento = n;
                i ++;
                AnteproyectoFC.elemento[i] = n;
            }
            if ((SistemaFC.existe[n] == 1) && (SistemaFC.tipoE[n] == 1))
                AnteproyectoFC.numUltimaLente = n;
        }



        AnteproyectoFC.numElementos = i;

        AnteproyectoFC.elemento[i+1] = 7;

        if (i < 1)
            return 0;       //  No hay elementos definidos


        return 1;
    }


    //=====================     tablaTramos     ============================

    //  Crea la tabla de tramos para los rayos

    static int     tablaTramos (int obj)
    {
        int     i, j = 0, n;

        if ((SistemaFC.real[obj] == 0) ||        //  Es objeto virtual
            (SistemaFC.zObjeto[obj] >= 1.e10))    //  Objeto en el infinito
        {
            for (i = 0; i <= AnteproyectoFC.numElementos; i ++)
                AnteproyectoFC.tramo[obj][i] = AnteproyectoFC.elemento[i];
            AnteproyectoFC.numTramos[obj] = AnteproyectoFC.numElementos + 1;
            AnteproyectoFC.tramo[obj][i] = 7;
        }
        else                                //  Es objeto real
        {
            AnteproyectoFC.tramo[obj][0] = 0;

                            //  Toma el primer elemento situado a su derecha
            for (i = 1; i <= AnteproyectoFC.numElementos; i ++)
            {
                n = AnteproyectoFC.elemento[i];
                if (SistemaFC.zObjeto[obj] >= SistemaFC.valZE[n])
                {
                    continue;
                }
                j ++;
                AnteproyectoFC.tramo[obj][j] = AnteproyectoFC.elemento[i];
            }

            AnteproyectoFC.numTramos[obj] = j+1;
            AnteproyectoFC.tramo[obj][j+1] = 7;
        }  



        if (AnteproyectoFC.numTramos[obj] < 2)
            return 0;       //  No hay tramos definidos


        return 1;
    }


    //=====================     marchaParaxial      ========================

    //  Calcula la posicin y tamao de las imgenes para cada elemento

    static int     marchaParaxial (int obj)
    {
        int     i, ia, k, n;
        double  z, y, yp, zE, s, fp, sp, zObj;

        n = AnteproyectoFC.numTramos[obj];

        if (n < 1)          //  No hay elementos definidos
            return 0;

        if (SistemaFC.existeObjeto[obj] == 0)    //  No est definido ese objeto;
            return 0;
        
                                        //  Datos del objeto
        
        zE = SistemaFC.valZE[AnteproyectoFC.tramo[obj][1]];
        AnteproyectoFC.zImag[obj][0] = zObj = SistemaFC.zObjeto[obj];
        AnteproyectoFC.yImag[obj][0] = SistemaFC.valYObj[obj] =  
                      (zE - zObj) * SistemaFC.campoDib[obj] * 3.141592 / 180./ 8.;


        for (k = 1; k <= n; k ++)
        {
            i = AnteproyectoFC.tramo[obj][k];
            ia = AnteproyectoFC.tramo[obj][k-1];
        
        
            if (SistemaFC.tipoE[i] == 0)        //  Es diafragma
            {
                AnteproyectoFC.zImag[obj][i] = AnteproyectoFC.zImag[obj][ia];
                AnteproyectoFC.yImag[obj][i] = AnteproyectoFC.yImag[obj][ia];
            }
            else                                    //  Es lente
            {
                z = AnteproyectoFC.zImag[obj][ia];
                y = AnteproyectoFC.yImag[obj][ia];
                s = z - SistemaFC.valZE[i]; 
                fp = 1000. / SistemaFC.valPE[i];
                if (s == - fp)
                {
                    sp = 1.e10;
                    yp = y * sp / s;
                }
                else
                {
                    sp = s * fp / (s + fp);
                    yp = y * fp / (s + fp);
                }
                AnteproyectoFC.zImag[obj][i] = ExactoFC.valZParax = SistemaFC.valZE[i] + sp;
                AnteproyectoFC.yImag[obj][i] = yp;
            }

        }
        
        AnteproyectoFC.zImag[obj][7] = 1600.;

        return 1;
    }



    //=====================     preparaRayos    =============================

    //  Calcula los coeficientes a, b, de los rayos en espacio objeto
    //  Utiliza las coordenadas verdaderas


    static int    preparapRayos (int obj)
    {
        int     i, j, n = 1, tipo;
        double  pot, zObj, yObj, zE, h, hmax, sigma, a, b;
        double  f, tg, tgMax = .5;


        zObj = AnteproyectoFC.zImag[obj][0];
        yObj = AnteproyectoFC.yImag[obj][0];


                            //  Primer elemento que existe

        n = AnteproyectoFC.tramo[obj][1];
        if (n > 6)
        {
            AnteproyectoFC.pintarRayos[obj] = 0;
            return 0;
        }

                            //  Toma el elemento n como pupila de entrada

        zE = SistemaFC.valZE[n];
        tipo = SistemaFC.tipoE[n];
        pot = SistemaFC.valPE[n];
        hmax = SistemaFC.valDE[n] / 2.;

        if (hmax == 0.)
        {
            AnteproyectoFC.pintarRayos[obj] = 0;
            return 0;
        }

        if ((pot == 0.) && (tipo == 1))
        {
            AnteproyectoFC.pintarRayos[obj] = 0;
            return 0;
        }

        if ((hmax == AnteproyectoFC.diamMax/2.) && (tipo == 0))
        {
            AnteproyectoFC.pintarRayos[obj] = 0;
            return 0;
        }


                            //  Limita el ngulo de abertura
        tg = Math.abs (hmax / (zE - zObj));
        if (tg > tgMax)
            f = tgMax / tg;
        else
            f = 1.;
        hmax = hmax * f;     //  Radio de la primera abertura (Hace de pupila) 


                            //  Traza 11 rayos de eje y 11 de campo
                            //  en espacio objeto.

        for (i = 0; i < 11; i ++)
        {
                                    //  Altura en elemento n
            h = (i - 5) * hmax / 5.5;

                                        //  Rayos de eje
            AnteproyectoFC.apRayo[obj][0][i] = a = h / (zE - zObj);
            AnteproyectoFC.bpRayo[obj][0][i] = h - zE * a;
            AnteproyectoFC.zpRayo[obj][0][i] = 0.;
            AnteproyectoFC.ypRayo[obj][0][i] = AnteproyectoFC.bpRayo[obj][0][i];
            AnteproyectoFC.hRayo[obj][n][i] = h;
            AnteproyectoFC.pasa[obj][0][i] = 1;

                                        //  Rayos de campo
            AnteproyectoFC.apRayo[obj][0][i+11] = a = (h - yObj) / (zE - zObj);
            AnteproyectoFC.bpRayo[obj][0][i+11] = h - zE * a;
            AnteproyectoFC.zpRayo[obj][0][i+11] = 0.;
            AnteproyectoFC.ypRayo[obj][0][i+11] = AnteproyectoFC.bpRayo[obj][0][i+11];
            AnteproyectoFC.hRayo[obj][n][i+11] = h;
            AnteproyectoFC.pasa[obj][0][i+11] = 1;
        }

        return 1;
    }



    //=======================   coeficientesRayos   ===========================


    //  Calcula los coeficientes a, b de cada rayo 


    static int  coeficientesRayos (int obj)
    {
        int     i, ip, ia, j, k, n;
        double  a, b, zE, ap, bp, zEp, z, y, zp, yp, h, hp;

        n = AnteproyectoFC.numTramos[obj];


                        //  Prepara los rayos en el espacio objeto

        if (preparapRayos (obj) == 0)
        {
            AnteproyectoFC.pintarRayos[obj] = 0;
            return 0;
        }


        for (j = 0; j < 22; j ++)
        {
                        //  Calcula los coeficientes a, b de cada rayo
                        //  a la salida del elemento k

            for (k = 1; k < n; k ++)
            {
                i = AnteproyectoFC.tramo[obj][k];
                ia = AnteproyectoFC.tramo[obj][k-1];
                ip = AnteproyectoFC.tramo[obj][k+1];

                zE = SistemaFC.valZE[i];
                zEp = SistemaFC.valZE[ip];
                z = AnteproyectoFC.zImag[obj][i];
                if (z == zE)
                {
                    a = AnteproyectoFC.apRayo[obj][ia][j]; 
                    b = AnteproyectoFC.bpRayo[obj][ia][j]; 

                }
                else
                {
                    if (j < 11)
                        y = 0.;     //  Punto del eje del objeto
                                    //  Todas las imgenes del objeto estn en el eje
                    else
                                    //  Punto de borde del objeto 
                        y = AnteproyectoFC.yImag[obj][i]; 

                    h = AnteproyectoFC.hRayo[obj][i][j];
                   
                                    //  Comprueba si el rayo cruza el elemento
                   
                    if (Math.abs (h) <= SistemaFC.valDE[i] / 2.) 
                        AnteproyectoFC.pasa[obj][i][j] = 1;
                    else
                        AnteproyectoFC.pasa[obj][i][j] = 0;
                   
                    a = (y - h) / (z - zE);
                    b = y - z * a;
                }

                hp = a * zEp + b;          //   Altura en el elemento siguiente

                AnteproyectoFC.apRayo[obj][i][j] = a;
                AnteproyectoFC.bpRayo[obj][i][j] = b;
                AnteproyectoFC.hRayo[obj][ip][j] = hp;
            }

        }

        AnteproyectoFC.pintarRayos[obj] = 1;
        return 1;
    }




    //=======================   interseccionesRayos   =========================


    //  Calcula las intersecciones de los rayos con las superficies


    static int  interseccionesRayos (int obj)
    {
        int     i, ip, j, k, n;
        double  a, b, zE, ap, bp, zEp, z, y, zp, yp, h, hp;

        n = AnteproyectoFC.numTramos[obj];


        for (j = 0; j < 22; j ++)
        {

                        //  Calcula las intersecciones de cada rayo
                        //  a la salida del elemento k y entrada del k+1

            for (k = 0; k < n; k ++)
            {
                i = AnteproyectoFC.tramo[obj][k];
                ip = AnteproyectoFC.tramo[obj][k+1];

                a = AnteproyectoFC.apRayo[obj][i][j];
                b = AnteproyectoFC.bpRayo[obj][i][j];
                zE = SistemaFC.valZE[i];
                zEp = SistemaFC.valZE[ip];


                                //  Interseccin a la salida del elemento k

                if (i == 0)             //  Es el objeto
                {
                    AnteproyectoFC.zpRayo[obj][0][j] = zE;
                    AnteproyectoFC.ypRayo[obj][0][j] = AnteproyectoFC.hRayo[obj][0][j];
                }
                else
                {
                    Caja.doble1 = a;
                    Caja.doble2 = b;
                    Caja.doble3 = zE + SistemaFC.espesor[i] / 2.;
                    Caja.doble4 = - SistemaFC.radioDib[i];

                    if (rectaEsfera () == 0)
                    {
                        AnteproyectoFC.pasa[obj][i][j] = 0;
                    }

                    AnteproyectoFC.zpRayo[obj][i][j] = Caja.doble1;
                    AnteproyectoFC.ypRayo[obj][i][j] = Caja.doble2;
                }

                                //  Interseccin a la entrada del elemento k+1

                if (ip == 7)             //  Es el borde derecho
                {
                    AnteproyectoFC.zaRayo[obj][ip][j] = 1600;
                    AnteproyectoFC.yaRayo[obj][ip][j] = AnteproyectoFC.hRayo[obj][ip][j];
                }
                else
                {
                    Caja.doble1 = a;
                    Caja.doble2 = b;
                    Caja.doble3 = zEp - SistemaFC.espesor[ip] / 2.;
                    Caja.doble4 = SistemaFC.radioDib[ip];

                    if (rectaEsfera () == 0)
                    {
                        AnteproyectoFC.pasa[obj][ip][j] = 0;
                    }


                    AnteproyectoFC.zaRayo[obj][ip][j] = Caja.doble1;
                    AnteproyectoFC.yaRayo[obj][ip][j] = Caja.doble2;
                }
            }
        }

        return 1;
    }



    //====================  rectaEsfera     ===================================

    //  Calcula la interseccin de una recta con una esfera.
    //  Recibe los coeficientes a, b del rayo, el vrtice de la superficie
    //  y el radio de curvatura.
    //  Calcula las coordenadas del punto de interseccin. 
    //  Pone 0 en control de paso si el rayo se sale de la esfera


        //  Entrada:

    //  Caja.doble1 = Coeficiente a del rayo
    //  Caja.doble2 = Coeficiente b del rayo
    //  Caja.doble3 = Vrtice de la superficie
    //  Caja.doble4 = Radio de curvatura de la superficie

        //  Salida:

    //  Caja.doble1 = z de la interseccin
    //  Caja.doble2 = y de la interseccin
    //  Caja.int1 = control de paso


    static int     rectaEsfera ()
    {
        int     control;
        double  a, b, zv, zc, r, A, B, C, x1, x2, z, y, rad, raiz;

                        //  lee los datos

        a = Caja.doble1;
        b = Caja.doble2;
        zv = Caja.doble3;
        r = Caja.doble4;


        if (Math.abs (r) > 1.e8)        //  Superficie plana
        {
            z = zv;
            control = 1;       
        }
        else                            //  Superficie esfrica
        {
                            //  Halla la posicin del centro
    
            zc = zv + r;
    
                            //  Clculo de la interseccin
    
            A = a * a + 1;
            B = a * b - zc;
            C = zc * zc + b * b - r * r;
    
            if ((rad = B * B - A * C) < 0.)
            {
                z = zv;
                control = 0;              //  El rayo se sale de la esfera
            }
            else
            {
                raiz = Math.sqrt (rad);
        
                x1 = (- B + raiz) / A;
                x2 = (- B - raiz) / A;
        
                if (r > 0)
                {
                    z = x2;
                }
                else
                {
                    z = x1;
                }
                control = 1;
            }
        }

        if (Math.abs (a) < 1.e-5)
            y = b;
        else
            y = a * z + b;

                //  Salida

        Caja.doble1 = z;
        Caja.doble2 = y;
        Caja.int1 = control;

        return control;
    }


    //=====================     calculaPupilas ()   ===========================


    //  Funcin para hallar el diafragma de abertura y las pupilas
    //  para el objeto 1


    static int calculaPupilas ()
    {
        int     i, ia, k, n, j, na;
        int     nDA = 1, nRDA = 1;
        int     numPrimerElemento = AnteproyectoFC.elemento[1];

        double  r, rinf = 0.;
        double  s, sp, y, yp, fp, da;




                //  Halla el diafragma de abertura

        AnteproyectoFC.numDiafragma = AnteproyectoFC.elemento[1];

        for (k = 1; k <= AnteproyectoFC.numElementos; k ++)   
        {
            n = AnteproyectoFC.elemento[k];
            r = Math.abs (AnteproyectoFC.hRayo[0][n][6]) / SistemaFC.valDE[n];
            if (r > rinf)
            {
                rinf = r;
                nDA = AnteproyectoFC.numDiafragma = n;
                nRDA = AnteproyectoFC.numRelDiafragma = k;
            }
        }


                //  Halla la pupila de entrada

                        //  Datos del D.A.
              
        k = nRDA;                           //  n relativo del diafragma
        n = AnteproyectoFC.elemento[k];       //  n absoluto del diafragma

        sp = 0.;                //  El diafragma es imagen de s mismo
        yp = SistemaFC.valDE[nDA];


        for (k = nRDA - 1; k > 0; k --)   
        {
            n = AnteproyectoFC.elemento[k];          
            na = AnteproyectoFC.elemento[k+1];
            da = SistemaFC.valZE[na] - SistemaFC.valZE[n]; 
        
            s = sp - da;             //  Frmulas de acoplamiento
            y = yp;
        
            if (SistemaFC.tipoE[n] == 0)        //  Es diafragma
            {
                sp = s;
                yp = y;
            }
            else                                    //  Es lente
            {
                fp = 1000. / SistemaFC.valPE[n];
                if (s == - fp)
                {
                    sp = 1.e10;
                    yp = y * sp / s;
                }
                else
                {
                    sp = s * fp / (s + fp);
                    yp = y * fp / (s + fp);
                }
            }
        }

        AnteproyectoFC.zPE = SistemaFC.valZE[n] - sp;
        AnteproyectoFC.DPE = Math.abs (yp);


                //  Halla la pupila de salida

                        //  Datos del D.A.
              
        k = nRDA;                           //  n relativo del diafragma
        n = AnteproyectoFC.elemento[k];       //  n absoluto del diafragma

        sp = 0.;                //  El diafragma es imagen de s mismo
        yp = SistemaFC.valDE[nDA];


        for (k = nRDA + 1; k <= AnteproyectoFC.numElementos; k ++)   
        {
            n = AnteproyectoFC.elemento[k];          
            na = AnteproyectoFC.elemento[k-1];
            da = SistemaFC.valZE[n] - SistemaFC.valZE[na]; 
        
            s = sp - da;             //  Frmulas de acoplamiento
            y = yp;
        
            if (SistemaFC.tipoE[n] == 0)        //  Es diafragma
            {
                sp = s;
                yp = y;
            }
            else                                    //  Es lente
            {
                fp = 1000. / SistemaFC.valPE[n];
                if (s == - fp)
                {
                    sp = 1.e10;
                    yp = y * sp / s;
                }
                else
                {
                    sp = s * fp / (s + fp);
                    yp = y * fp / (s + fp);
                }
            }
        }

        AnteproyectoFC.zPS = SistemaFC.valZE[n] + sp;
        AnteproyectoFC.DPS = Math.abs (yp);


        return 1;
    }

}
