/**************************************************************************\
*                                                                          *
*                   Clculo y dibujo de sistemas pticos                   *
*                                                                          *
*                  Variante para sistemas de focal corta                   *
*                                                                          *
*   Para el clculo utiliza los mismos valores del destinado a focal larga *
*   (SistemaFL) modificando la unidad en la relacin 2.8, lo que se pone   *
*   de manifiesto slo en la escritura.                                    *
*   Como esto haca que los incrementos en las correderas fueran de 0.28   *
*   se hizo un nuevo cambio multiplicando los valores en las correderas    *
*   por 2.8 para que los saltos fueran de 0.1                              *
*                                                                          *
\**************************************************************************/



import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.applet.*;


//==================        Clase principal     ===============================


public class SistemaFC extends JPanel
{
    static SistemaFC    sistema;

    static PanelBaseFC  panelBase;
    AnteproyectoFC      anteproyecto;
    ExactoFC            exacto;
    static int          lang = 0;
    static String       strTeoria = null;

    static  DibujoEscalaHFC     canvasEscalaH;
    static  DibujoEscalaVFC     canvasEscalaV;

    static int ancho;
    static int alto;        

    static double  factorFO = 2.8;  // Cambio de escalas para fotografa y ojo

    static int ultimoHecho = 0;     //  0 si no se ha calculado nada
                                    //  1 si se ha calculado "Anteproyecto"
                                    //  2 si se ha calculado "Exacto"

                        //  Datos generales del objeto

    static  int     existeObjeto[] = {0, 0};
    static  int     real[] = {1, 1};
    static  double  valZObj[] = {-1600, -1600};   // Posicin dada por la corredera
    static  double  zObjeto[] = {-1600, -1600};   //    "   utilizada para clculo
    static  double  valYObj[] = {0., 0.};
    static  double  campo[] = {0., 0.};
    static  double  campoDib[] = {0., 0.};


                //  Datos de los elementos (lentes o diafragmas)

                    //  El subndice 0 corresponde al objeto
                    //  El subndice 7 corresponde al borde derecho del cuadro


    static int tipoE[] = new int[8];          //  1 si es lente, 0 si es diafragma
    static int existe[] = new int[8];         //  1 si existe, 0 si no existe
    static double valZE[] = new double[8];    //  Posicin
    static double valPE[] = new double[8];    //  Potencia 
    static double valFE[] = new double[8];    //  Focal
    static double valDibPE[] = new double[8]; //  Potencia adaptada al dibujo
    static double valDE[] = new double[8];    //  Dimetro del diafragma
    static double curv[] = new double[8];     //  Curvatura
    static double radio[] = new double[8];    //  Radio de curvatura
    static double radioDib[] = new double[8]; //  Radio de curvatura para dibujo
    static double espesor[] = new double[8];  //  Espesor de la lente en eje
    static double altura[] = new double[8];   //  Radio de la abertura
    static double dist[] = new double[8];     //  Distancia entre los elementos 
                                              //  "i" e "i+1".
                                              //  Se toma igual a la mitad de la
                                              //  suma de ambos espesores.
                                              //  En las lentes el espesor es
                                              //  el doble de la potencia de dibujo.


    //==========    Bloque con todos los textos de este fichero     ===========

    static String text[][] =
    {
        { "TRAAT DE RAIGS", "TRAZADO DE RAYOS", "RAY TRACING" },
        { "Avantprojecte", "Anteproyecto", "Preliminary sketch" },
        { "Clcul exacte", "Clculo exacto", "Exact calculation" },
        { "Llegir un fitxer", "Leer un fichero", "Read file" },
        { "Enregistrar", "Grabar un fichero", "Save to file" },
        { "Resum de teoria", "Resumen de teora", "Theory summary" },
        { "Sortida", "Salida", "Exit" },
        { "TEORIA", "TEORIA", "THEORY" },
    };
  

    //=================         Se inica la aplicacin        ===================

    public SistemaFC ()

    {
        lang = Sistema.lang;
        ancho = Sistema.ancho;
        alto = Sistema.altoDibujo;
        strTeoria = Sistema.teoria[lang];

                        //  Crea el contenido de la ventana: "panelBase"

        setSize (ancho, alto);
        setLayout (null);

                        //  Crea el panel base para el "applet"

        panelBase = new PanelBaseFC ();
        add (panelBase);
  
         
                        //  Crea las otras clases que luego necesitar

        anteproyecto = new AnteproyectoFC ();
        exacto = new ExactoFC ();

                        //  Panel para la escala vertical 

        canvasEscalaV = new DibujoEscalaVFC ();
        canvasEscalaV.setBackground (Color.black);
        canvasEscalaV.setBounds (0, 0, 30, 210);
        add (canvasEscalaV);
        canvasEscalaV.repaint ();

                        //  Panel para la escala horizontal 

        canvasEscalaH = new DibujoEscalaHFC ();
        canvasEscalaH.setBackground (Color.black);
        canvasEscalaH.setBounds (0, 210, 800, 20);
        add (canvasEscalaH);
        canvasEscalaH.repaint ();

        repaint ();
    }

}


//==================        Clase marco         ===============================


class MarcoFC extends JFrame 
{
    public  MarcoFC (String nombre)
    {
        super (nombre);
        setResizable (false);
        setDefaultCloseOperation (EXIT_ON_CLOSE);
    }
}


//===================   Clase PanelBase     ===================================

class PanelBaseFC extends JPanel implements ActionListener
{

    static  JPanel   panelMP;


    //-----------------  Crea varias fuentes    -------------------------------

    Font fuente1 = new Font ("Dialog", Font.BOLD, 24);
    Font fuente2 = new Font ("Dialog", Font.BOLD, 12);

    Color   colorTexto1 = Color.white;
    Color   colorTexto2 = Color.black;



    //=====================  Constructor  =====================================


    public PanelBaseFC ()
    {

        setLayout (null);
        setSize (800, 450);
            


        //==============  Panel para el men principal  ======================

        panelMP = new JPanel ();
        panelMP.setLayout (null);                 
        panelMP.setBackground (Color.gray);
        panelMP.setBounds (0, 220, 800, 230);

                    //  Ttulo

        JLabel labelAntep = new JLabel (SistemaFC.text[0][SistemaFC.lang]);
        labelAntep.setBounds (50, 10, 200, 30);
        labelAntep.setForeground (colorTexto1);
        labelAntep.setFont (fuente2);
        panelMP.add (labelAntep);

                    //  Prepara los botones del men principal

                        //  Para el anteproyecto

        JButton buttonAntep = new JButton (SistemaFC.text[1][SistemaFC.lang]);
        buttonAntep.setBounds (10, 40, 200, 30);
        buttonAntep.setForeground (colorTexto2);
        buttonAntep.setFont (fuente2);

                                //  Para que responda a una pulsacin
        buttonAntep.addActionListener (this);  

                                //  El texto que devuelve cuando responde
        buttonAntep.setActionCommand ("buttonAntep");
        panelMP.add (buttonAntep);


                        //  Para el clculo excato

        JButton buttonExacto = new JButton (SistemaFC.text[2][SistemaFC.lang]);
        buttonExacto.setBounds (10, 70, 200, 30);
        buttonExacto.setForeground (colorTexto2);
        buttonExacto.setFont (fuente2);
        buttonExacto.addActionListener (this);  
        buttonExacto.setActionCommand ("buttonExacto");
        panelMP.add (buttonExacto);


                        //  Para leer un fichero

        JButton buttonLeer = new JButton (SistemaFC.text[3][SistemaFC.lang]);
        buttonLeer.setBounds (10, 100, 200, 30);
        buttonLeer.setForeground (colorTexto2);
        buttonLeer.setFont (fuente2);
        buttonLeer.addActionListener (this);  
        buttonLeer.setActionCommand ("buttonLeer");
        panelMP.add (buttonLeer);


                        //  Para grabar un fichero

        JButton buttonGrabar = new JButton (SistemaFC.text[4][SistemaFC.lang]);
        buttonGrabar.setBounds (10, 130, 200, 30);
        buttonGrabar.setForeground (colorTexto2);
        buttonGrabar.setFont (fuente2);
        buttonGrabar.addActionListener (this);  
        buttonGrabar.setActionCommand ("buttonGrabar");
        panelMP.add (buttonGrabar);


                        //  Para la pgina de teora

        JButton buttonTeoria = new JButton (SistemaFC.text[5][SistemaFC.lang]);
        buttonTeoria.setBounds (10, 160, 200, 30);
        buttonTeoria.setForeground (colorTexto2);
        buttonTeoria.setFont (fuente2);
        buttonTeoria.addActionListener (this);  
        buttonTeoria.setActionCommand ("buttonTeoria");
        panelMP.add (buttonTeoria);


                        //  Para el botn "Salida"

        JButton salida = new JButton (SistemaFL.text[6][SistemaFL.lang]);
        salida.setBounds (10, 190, 200, 30);
        salida.setForeground (colorTexto2);
        salida.setFont (fuente2);
        salida.addActionListener (this);  
        salida.setActionCommand ("buttonSalida");
        panelMP.add (salida);


        //=====================================================================


        //------------   Se aade el panel del men a la ventana  --------------

        add (panelMP);
    }


    //========  Define los mtodos de ActionListener  =========================

    public void actionPerformed (ActionEvent e) 
    {
        int i, n;
        double dato[] = new double[8];



            //  Obtiene la etiqueta del botn pulsado

        String s = e.getActionCommand ();


        //----------------      Llama a Anteproyecto        -------------------

        if (s == "buttonAntep")  
        {
            remove (panelMP);
            AnteproyectoFC.abrir ();
            repaint ();
        }


        //--------------        Llama a Clculo Exacto      -------------------

        if (s == "buttonExacto") 
        {
            remove (panelMP);
            ExactoFC.abrir ();
            repaint ();
        }

        //--------------         Llama a Leer Fichero       -------------------

        if (s == "buttonLeer") 
        {
                            //      Crea un selector de ficheros

            FileDialog selector = new FileDialog (new Frame(), "Selector");
            selector.setForeground (Color.black);
            selector.setFont (fuente2);
            selector.show ();
        
            String fichero = selector.getFile ();
            String dir = selector.getDirectory ();
            String dirFichero = dir + fichero;

            if ((dir == null) || (fichero == null))
            {
                            //  No se ha elegido ningn fichero
            
                return;
            }


            try
            {
                FileReader file = new FileReader (dirFichero);  
                BufferedReader lector = new BufferedReader (file);

                                //  Lee los datos del objeto 1

                n = leeLineaAscii (lector, dato, 5);
                if (n < 0)       //  Error de lectura
                {
                    lector.close ();
                    return;
                }

                                //  Convierte los datos;

                SistemaFC.existeObjeto[0] = (int) dato[0]; 
                SistemaFC.real[0]         = (int) dato[1]; 
                SistemaFC.valZObj[0]      = dato[2]; 
                SistemaFC.campo[0]        = dato[3]; 

                                //  Lee los datos del objeto 2

                n = leeLineaAscii (lector, dato, 5);
                if (n < 0)       //  Error de lectura
                {
                    lector.close ();
                    return;
                }

                                //  Convierte los datos;

                SistemaFC.existeObjeto[1] = (int) dato[0]; 
                SistemaFC.real[1]         = (int) dato[1]; 
                SistemaFC.valZObj[1]      = dato[2]; 
                SistemaFC.campo[1]        = dato[3]; 

                                //  Inicia los datos de las lentes

                for (i = 1; i < 7; i ++)
                {
                    SistemaFC.existe[i] = 0;
                    SistemaFC.tipoE[i]  = 1;
                    SistemaFC.valPE[i]  = 0;
                    SistemaFC.valDE[i]  = 70;
                    AnteproyectoFC.datosCorrederaHLente (i);
                }   

                                //  Lee los datos de las lentes

                for (i = 1; i < 7; i ++)
                {
                    n = leeLineaAscii (lector, dato, 8);
          
                    if (n == - 1)       //  Final del fichero
                    {
                        lector.close ();
                        break;
                    }
          
                    if (n == - 2)       //  Error de lectura
                    {
                        lector.close ();
                        return;
                    }

                            //  Convierte los datos;

                    SistemaFC.existe[i] = (int) dato[0];
                    SistemaFC.tipoE[i]  = (int) dato[1];
                    SistemaFC.valZE[i]  = dato[2];
                    SistemaFC.valPE[i]  = dato[3];
                    SistemaFC.valDE[i]  = dato[4];
                    ExactoFC.valBE[i]  = dato[5];
                    ExactoFC.indice[i] = dato[6]; 
                    ExactoFC.abbe[i]   = dato[7]; 

                }   
            }

            catch (IOException ioe)
            {
                            //  Error de lectura 
            
                return;
            }     

                            //  Llama a "Anteproyecto"

            remove (panelMP);
            AnteproyectoFC.abrir ();
            repaint ();
        }


        //--------------         Llama a Grabar Fichero       -------------------

        if (s == "buttonGrabar") 
        {

            if (SistemaFC.existeObjeto[0] == 0)
                return;


                            //    Crea un selector de ficheros

            FileDialog selector = new FileDialog (new Frame(), "Selector");
            selector.setForeground (Color.black);
            selector.setFont (fuente2);
            selector.show ();
        
            String fichero = selector.getFile ();
            String dir = selector.getDirectory ();
            String dirFichero = dir + fichero;

            if ((dir == null) || (fichero == null))
            {
                            //  No se ha elegido ningn fichero
            
                return;
            }

                                //  Empieza la grabacin

            try
            {
                FileWriter file = new FileWriter (dirFichero);  
                BufferedWriter grabar = new BufferedWriter (file);


                                    //  Graba el objeto 1

                String objeto = SistemaFC.existeObjeto[0]+"  "+ 
                                SistemaFC.real[0]+"  "+    
                                SistemaFC.valZObj[0]+"  "+
                                SistemaFC.campo[0];  

                grabar.write (objeto);
                grabar.newLine ();

                                    //  Graba el objeto 2 (del Anteproyecto)

                objeto = SistemaFC.existeObjeto[1]+"  "+ 
                         SistemaFC.real[1]+"  "+    
                         SistemaFC.valZObj[1]+"  "+
                         SistemaFC.campo[1];  

                grabar.write (objeto);
                grabar.newLine ();


                for (i = 1; i < 7; i ++)
                {
                    String lista = SistemaFC.existe[i]+"  "+ 
                                   SistemaFC.tipoE[i]+"  "+
                                   SistemaFC.valZE[i]+"  "+ 
                                   SistemaFC.valPE[i]+"  "+ 
                                   SistemaFC.valDE[i]+"  "+
                                   ExactoFC.valBE[i]+"  "+
                                   ExactoFC.indice[i]+"  "+
                                   ExactoFC.abbe[i];


                    grabar.write (lista);
                    grabar.newLine ();
                }
                grabar.close ();
            }

            catch (IOException ioe)
            {
                            //  Error de grabacin
                return;
            }     

        }

        //--------------         Llama a Leer Texto de Teora       -----------

        if (s == "buttonTeoria") 
        {
                            //      Crea un cuadro para texto

            JFrame teoria = new JFrame (SistemaFC.text[7][SistemaFC.lang]);
            teoria.setForeground (Color.black);
            teoria.setFont (fuente2);
            teoria.setSize (700, 500);
        
            JTextPane texto = new JTextPane ();
            teoria.getContentPane().add (texto);
            texto.setBackground (new Color (204, 204, 204));

            JScrollPane scroll = new JScrollPane (texto);
            scroll.setBackground (new Color (204, 204, 204));
            teoria.getContentPane().add (scroll);

            teoria.show ();

            java.net.URL    info_page = null;

            try
            {
                info_page = getClass ().getResource (SistemaFC.strTeoria);
            }
            catch (Exception eee)
            {
                System.err.println ("Error al cargar el texto");
            }

            try
            {
                texto.setPage (info_page);
            }
            catch (IOException  ee)
            {
                System.err.println ("Error al leer una URL mala");
            }
        }

        //--------------         Salida                      -----------

        if (s == "buttonSalida") 
        {
            System.exit (0);
        }
    }



    /**********************************************************************\
    *                        LeeLineaAscii                                 *
    *              Lee una lnea de datos de un fichero ASCII              *
    *               Devuelve el n de datos ledos                         *
    *               Al final del fichero devuelve - 1                      *
    \**********************************************************************/


    static int leeLineaAscii (BufferedReader lector, double dato[], int nMax)
    {

        int    i, nchar = 0;
        int    numDatos = 0;
        char   cadenaBase[] = new char[100];
        char   cadena[] = new char[100];
        String lineaInicial; 
        String lineaBase;  //  Para toda la lnea pero sin ceros extremos
        String trozo;      //  Para cada trozo

        try
        {
                    //  Lee una lnea del fichero
        
            lineaInicial = lector.readLine ();
            if (lineaInicial == null)     //   Final del fichero
            {
                return - 1;
            }


            while (true)
            {

                        //  Le quita los ceros iniciales
            
                lineaBase = lineaInicial.trim ();
          
                        //  La convierte en cadena de caracteres
            
                cadenaBase = lineaBase.toCharArray ();
            
                if (cadenaBase.length < 1)
                    break;
    
                        //  Copia la primera parte de la cadena
                        //  (El primer nmero ASCII)
            
                for (i = 0; i < cadenaBase.length; i ++)
                {
                    if (Character.isWhitespace (cadenaBase[i])) 
                        break;
                    cadena[i] = cadenaBase[i];
                }
                nchar = i;
            
                        //  La convierte en "String"
            
                trozo = new String (cadena, 0, nchar);
            
                        //  Extrae el n entero que representa
            
                dato[numDatos] = Double.parseDouble (trozo);
                numDatos ++;
                if (numDatos > nMax)
                    break;
    
                        //  Crea una nueva "String" con la 2 parte
                        //  de la cadena
            
                if (cadenaBase.length <= nchar)
                    break;
                lineaInicial = new String 
                          (cadenaBase, nchar, cadenaBase.length-nchar);

            }

        }
        catch (EOFException  eof)
        {
                //  Error de lectura 
        
            return - 2;
        }
        catch (IOException ioe)
        {
            return - 2;
        }     

        return numDatos;
    }    

}
                                                                                                                                                                
